@@ -1,7 +1,7 @@
 /*
  *  Authors: Rodney Dawes <dobey@ximian.com>
  *
- *  Copyright 2003-2004 Novell, Inc. (www.novell.com)
+ *  Copyright 2003-2005 Novell, Inc. (www.novell.com)
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of version 2 of the GNU General Public License
@@ -61,7 +61,7 @@ GnomeWPItem * gnome_wp_item_new (const gchar * filename,
     item->options = gconf_client_get_string (client, WP_OPTIONS_KEY, NULL);
 
     if (!strcmp (item->options, "none")) {
-      item->options = g_strdup ("wallpaper");
+      item->options = g_strdup ("scaled");
     }
     gnome_wp_item_update_description (item);
      
@@ -71,6 +71,8 @@ GnomeWPItem * gnome_wp_item_new (const gchar * filename,
     item = NULL;
   }
 
+  g_object_unref (client);
+
   return item;
 }
 
@@ -281,10 +283,12 @@ void gnome_wp_item_update_description (GnomeWPItem * item) {
   } else {
     gchar * info;
 
-    info = g_strdup_printf (_("%s, %d x %d pixels"),
+    info = g_strdup_printf ("%s, %d %s x %d %s",
 			    gnome_vfs_mime_get_description (item->fileinfo->mime_type),
 			    item->width,
-			    item->height);
+			    ngettext ("pixel", "pixels", item->width),
+			    item->height,
+			    ngettext ("pixel", "pixels", item->height));
 
     item->description = g_markup_printf_escaped ("<b>%s</b>\n"
 						 "%s",
