@@ -26,16 +26,15 @@
 # include "config.h"
 #endif
 
-//#include <glib/gstrfuncs.h>
+#include <glib/gi18n.h>
 #include <gdk/gdk.h>
 #include <gdk/gdkx.h>
 #include <gconf/gconf-client.h>
+#include "gnome-settings-xmodmap.h"
 
 #include <string.h>
 #include <time.h>
 
-#include <libgnome/gnome-i18n.h>
-
 #include <libxklavier/xklavier.h>
 #include <libxklavier/xklavier_config.h>
 #include <libgswitchit/gswitchit_config.h>
@@ -54,8 +53,10 @@ static gboolean initedOk;
 static PostActivationCallback paCallback = NULL;
 static void *paCallbackUserData = NULL;
 
-static const char DISABLE_XMM_WARNING_KEY[] =
-    "/desktop/gnome/peripherals/keyboard/disable_xmm_and_xkb_warning";
+static const char KNOWN_FILES_KEY[] =
+    "/desktop/gnome/peripherals/keyboard/general/known_file_list";
+static const char DISABLE_SYSCONF_CHANGED_WARNING_KEY[] =
+    "/desktop/gnome/peripherals/keyboard/general/disable_sysconfig_changed_warning";
 
 typedef enum {
 	RESPONSE_USE_X,
@@ -82,26 +83,32 @@ gnome_settings_keyboard_log_appender (const char file[], const char function[],
 static void
 activation_error (void)
 {
-	char *vendor = ServerVendor (GDK_DISPLAY ());
+	char const *vendor = ServerVendor (GDK_DISPLAY ());
 	int release = VendorRelease (GDK_DISPLAY ());
-	gboolean badXFree430Release = (!strcmp (vendor,
-						"The XFree86 Project, Inc"))
+	gboolean badXFree430Release = (vendor != NULL)
+	    && (0 == strcmp (vendor, "The XFree86 Project, Inc"))
 	    && (release / 100000 == 403);
 
-	GtkWidget *msg = gtk_message_dialog_new_with_markup (NULL,
+	GtkWidget *dialog;
+	
+	/* VNC viewers will not work, do not barrage them with warnings */
+	if (NULL != vendor && NULL != strstr (vendor, "VNC"))
+	    return;
+
+	dialog = gtk_message_dialog_new_with_markup (NULL,
 							     0,
 							     GTK_MESSAGE_ERROR,
 							     GTK_BUTTONS_CLOSE,
 							     _
 							     ("Error activating XKB configuration.\n"
-							      "It can have under various circumstances:\n"
+							      "It can happen under various circumstances:\n"
 							      "- a bug in libxklavier library\n"
 							      "- a bug in X server (xkbcomp, xmodmap utilities)\n"
-							      "- X server with incompatible libxkbfile implemenation\n\n"
+							      "- X server with incompatible libxkbfile implementation\n\n"
 							      "X server version data:\n%s\n%d\n%s\n"
 							      "If you report this situation as a bug, please include:\n"
-							      "- The result of <b>xprop -root | grep XKB</b>\n"
-							      "- The result of <b>gconftool-2 -R /desktop/gnome/peripherals/keyboard/kbd</b>"),
+							      "- The result of %s\n"
+							      "- The result of %s"),
 							     vendor,
 							     release,
 							     badXFree430Release
@@ -110,10 +117,12 @@ activation_error (void)
 							     ("You are using XFree 4.3.0.\n"
 							      "There are known problems with complex XKB configurations.\n"
 							      "Try using simpler configuration or taking more fresh version of XFree software.")
-							     : "");
-	g_signal_connect (msg, "response",
+							     : "",
+							     "<b>xprop -root | grep XKB</b>",
+							     "<b>gconftool-2 -R /desktop/gnome/peripherals/keyboard/kbd</b>");
+	g_signal_connect (dialog, "response",
 			  G_CALLBACK (gtk_widget_destroy), NULL);
-	gtk_widget_show (msg);
+	gtk_widget_show (dialog);
 }
 
 static void
@@ -172,6 +181,9 @@ gnome_settings_keyboard_xkb_sysconfig_changed_response (GtkDialog * dialog,
 							SysConfigChangedMsgResponse
 							what2do)
 {
+	gboolean dontShowAgain = gtk_toggle_button_get_active (
+		GTK_TOGGLE_BUTTON (g_object_get_data (G_OBJECT (dialog), "chkDontShowAgain")));
+
 	switch (what2do) {
 	case RESPONSE_USE_X:
 		GSwitchItKbdConfigSaveToGConf (&initialSysKbdConfig);
@@ -180,6 +192,13 @@ gnome_settings_keyboard_xkb_sysconfig_changed_response (GtkDialog * dialog,
 		/* Do absolutely nothing - just keep things the way they are */
 		break;
 	}
+
+	if (dontShowAgain) {
+		GConfClient *confClient;
+		confClient = gconf_client_get_default ();
+		gconf_client_set_bool (confClient, DISABLE_SYSCONF_CHANGED_WARNING_KEY, TRUE, NULL);
+		g_object_unref (confClient);
+	}
 	gtk_widget_destroy (GTK_WIDGET (dialog));
 }
 
@@ -188,13 +207,14 @@ gnome_settings_keyboard_xkb_analyze_sysconfig (void)
 {
 	GConfClient *confClient;
 	GSwitchItKbdConfig backupGConfKbdConfig;
-	gboolean isConfigChanged;
+	gboolean isConfigChanged, dontShow;
 
 	if (!initedOk)
 		return;
 	confClient = gconf_client_get_default ();
 	GSwitchItKbdConfigInit (&backupGConfKbdConfig, confClient);
 	GSwitchItKbdConfigInit (&initialSysKbdConfig, confClient);
+	dontShow = gconf_client_get_bool (confClient, DISABLE_SYSCONF_CHANGED_WARNING_KEY, NULL);
 	g_object_unref (confClient);
 	GSwitchItKbdConfigLoadFromGConfBackup (&backupGConfKbdConfig);
 	GSwitchItKbdConfigLoadFromXInitial (&initialSysKbdConfig);
@@ -204,80 +224,113 @@ gnome_settings_keyboard_xkb_analyze_sysconfig (void)
 	    !GSwitchItKbdConfigEquals (&initialSysKbdConfig, &backupGConfKbdConfig);
 	/* config was changed!!! */
 	if (isConfigChanged) {
-		GtkWidget *msg = gtk_message_dialog_new_with_markup (NULL,
-								     0,
-								     GTK_MESSAGE_INFO,
-								     GTK_BUTTONS_NONE,
+		if (dontShow) {
+			g_warning ("The system configuration changed - but we remain silent\n");
+		} else {
+			GtkWidget *chkDontShowAgain = gtk_check_button_new_with_mnemonic (_ ("Do _not show this warning again"));
+			GtkWidget *alignDontShowAgain = gtk_alignment_new (0.5, 
+									   0.5, 
+									   0, 
+									   0);
+			GtkWidget *msg = gtk_message_dialog_new_with_markup (NULL,
+									     0,
+									     GTK_MESSAGE_INFO,
+									     GTK_BUTTONS_NONE,
 /* !! temporary one */
-								     _
-								     ("The X system keyboard settings differ from your current GNOME "
-								      "keyboard settings.  Which set would you like to use?"));
-		gtk_dialog_add_buttons (GTK_DIALOG (msg),
-					_("Use X settings"),
-					RESPONSE_USE_X,
-					_("Use GNOME settings"),
-					RESPONSE_USE_GNOME, NULL);
-		gtk_dialog_set_default_response (GTK_DIALOG (msg),
-						 RESPONSE_USE_GNOME);
-		g_signal_connect (msg, "response",
-				  G_CALLBACK
-				  (gnome_settings_keyboard_xkb_sysconfig_changed_response),
-				  NULL);
-		gtk_widget_show (msg);
+									     _
+									     ("The X system keyboard settings differ from your current GNOME "
+									      "keyboard settings.  Which set would you like to use?"));
+			gtk_container_set_border_width (GTK_CONTAINER (alignDontShowAgain), 
+						        12);
+			gtk_container_add (GTK_CONTAINER (alignDontShowAgain), 
+					   chkDontShowAgain);
+			gtk_container_add (GTK_CONTAINER ((GTK_DIALOG (msg))->vbox), 
+					   alignDontShowAgain);
+
+			gtk_dialog_add_buttons (GTK_DIALOG (msg),
+						_("Use X settings"),
+						RESPONSE_USE_X,
+						_("Use GNOME settings"),
+						RESPONSE_USE_GNOME, NULL);
+			gtk_dialog_set_default_response (GTK_DIALOG (msg),
+							 RESPONSE_USE_GNOME);
+			g_object_set_data (G_OBJECT (msg), "chkDontShowAgain", chkDontShowAgain);
+			g_signal_connect (msg, "response",
+					  G_CALLBACK
+					  (gnome_settings_keyboard_xkb_sysconfig_changed_response),
+					  NULL);
+			gtk_widget_show_all (msg);
+		}
 	}
 	GSwitchItKbdConfigSaveToGConfBackup (&initialSysKbdConfig);
 	GSwitchItKbdConfigTerm (&backupGConfKbdConfig);
 }
 
-static void
-gnome_settings_keyboard_xkb_chk_lcl_xmm_response (GtkDialog * dlg,
-						  gint response)
+static gboolean 
+gnome_settings_chk_file_list (void)
 {
+	GDir *homeDir;
+	G_CONST_RETURN gchar *fname;
+	GSList *file_list = NULL;
+	GSList *last_login_file_list = NULL;
+	GSList *tmp = NULL;
+	GSList *tmp_l = NULL;
+	gboolean new_file_exist = FALSE;
 	GConfClient *confClient = gconf_client_get_default ();
-	switch (response) {
-	case GTK_RESPONSE_OK:
-		gconf_client_set_bool (confClient, DISABLE_XMM_WARNING_KEY, TRUE,
-				       NULL);
-		break;
+
+	homeDir = g_dir_open (g_get_home_dir (), 0, NULL);
+	while ((fname = g_dir_read_name (homeDir)) != NULL) {
+		if (g_strrstr (fname, "modmap")) {
+			file_list = g_slist_append (file_list, g_strdup (fname));
+		}
 	}
+	g_dir_close (homeDir);
+
+	last_login_file_list = gconf_client_get_list (confClient, KNOWN_FILES_KEY, GCONF_VALUE_STRING, NULL);
+
+	/* Compare between the two file list, currently available modmap files
+	   and the files available in the last log in */
+	tmp = file_list;
+	while (tmp != NULL) {
+		tmp_l = last_login_file_list;
+		new_file_exist = TRUE;
+		while (tmp_l != NULL) {
+			if (strcmp (tmp->data, tmp_l->data) == 0) {
+				new_file_exist = FALSE;
+				break;
+			}
+			else
+				tmp_l = tmp_l->next;
+		}
+		if (new_file_exist)
+			break;
+		else
+			tmp = tmp->next;
+	}
+
+	if (new_file_exist)
+		gconf_client_set_list (confClient, KNOWN_FILES_KEY, GCONF_VALUE_STRING, file_list, NULL);
+
+	g_slist_foreach (file_list, (GFunc) g_free, NULL);
+	g_slist_free (file_list);
+
+	g_slist_foreach (last_login_file_list, (GFunc) g_free, NULL);
+	g_slist_free (last_login_file_list);
+
 	g_object_unref (G_OBJECT (confClient));
-	gtk_widget_destroy (GTK_WIDGET (dlg));
+
+	return new_file_exist;
+
 }
 
 static void
 gnome_settings_keyboard_xkb_chk_lcl_xmm (void)
 {
-	GConfClient *confClient = gconf_client_get_default ();
-	gboolean disableWarning =
-	    gconf_client_get_bool (confClient, DISABLE_XMM_WARNING_KEY, NULL);
-	GDir *homeDir;
-	G_CONST_RETURN gchar *fname;
-	g_object_unref (G_OBJECT (confClient));
-	if (disableWarning)
-		return;
+	if (gnome_settings_chk_file_list ())
+		gnome_settings_modmap_dialog_call ();
+	gnome_settings_load_modmap_files ();
 
-	homeDir = g_dir_open (g_get_home_dir (), 0, NULL);
-	if (homeDir == NULL)
-		return;
-	while ((fname = g_dir_read_name (homeDir)) != NULL)
-		if (strlen (fname) >= 8
-		    && !g_ascii_strncasecmp (fname, ".xmodmap", 8)) {
-			GtkWidget *msg =
-			    gtk_message_dialog_new_with_markup (NULL, 0,
-								GTK_MESSAGE_WARNING,
-								GTK_BUTTONS_OK,
-								_
-								("You have a keyboard remapping file (%s) in your home directory whose contents will now be ignored."
-								 " You can use the keyboard preferences to restore them."),
-				fname);
-			g_signal_connect (msg, "response",
-					  G_CALLBACK
-					  (gnome_settings_keyboard_xkb_chk_lcl_xmm_response),
-					  NULL);
-			gtk_widget_show (msg);
-			break;
-		}
-	g_dir_close (homeDir);
+	
 }
 
 void
